/**
 DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER
 Copyright (c) 2010, R2AD, LLC
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the R2AD, LLC nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.cdmi.model;

import java.net.URI;
import java.util.List;

/**
 * A representation of a CDMI container in accordance with the CDMI spec.
 * Visit: snia.org/cloud for more information.
 * @author David
 */
public class CDMIContainer {

    //**********************************
    //MANDATORY FIELDS
    //**********************************

    /** URI of the container as specified in the request */
    protected URI objectURI;

    /** Object ID of the object */
    protected String objectID;

    /** URI for the parent object */
    protected URI parentURI;

    /** URI for the owning domain */
    protected URI domainURI;

    /** URI to the capabilities for the object.
     * The capabilities URI returned is based on the object type
     * and requested data system metadata fields
     */
    protected URI capabilitiesURI;

    /** A string indicating if the object is still in the process
     * of being created, and once the operation is complete, if
     * it was created successfully or an error occurred. The
     * value shall be the string "Processing", the string
     * "Complete", or an error message
     */
    protected String completionStatus;

    /** Metadata for the container. This field includes any user
     * and data system metadata specified in the request body
     * metadata field, along with storage system metadata
     * generated by the cloud storage system.
     */
    protected Object metadata;

    /** The range of the children returned in the children field */
    protected Object childrenRange;

    /** Names of the children objects in the container. Child
     * containers end with "/"
     */
    protected List children;

    //**********************************
    //OPTIONAL FIELDS
    //**********************************

    /** An integer numeric value from 0 through 100 */
    protected int percentComplete;

   /** A structure for each protocol enabled for this container */
    protected Object exports;

    /** URI(s) of the SnapShot containers */
    protected List snapShots;

    public CDMIContainer(URI objectURI, String objectID) {
        this.objectURI = objectURI;
        this.objectID = objectID;
    }

    public URI getObjectURI() {
        return objectURI;
    }

    public void setObjectURI(URI objectURI) {
        this.objectURI = objectURI;
    }

}
